@@ -153,6 +153,12 @@ public interface GroupManager {
 	 * @throws GroupRetrievalFault 
 	 */
 	List<GCubeGroup> listGroupsByUser(long userId) throws UserRetrievalFault, UserManagementSystemException, GroupRetrievalFault;
+	/**
+	 * * @return the list of user VREs
+	 * @throws UserManagementSystemException
+	 * @throws GroupRetrievalFault if the rootVO does not exists
+	 */
+	List<GCubeGroup> listVresByUser(long userId)  throws UserManagementSystemException, GroupRetrievalFault, UserRetrievalFault ;
 	/**
 	 * @param userId the LR userId
 	 * @param  serverName the host name of the server that is sending the request (e.g. i-marine.d4science.org)
@@ -237,4 +243,4 @@ public interface GroupManager {
 	 * @return a list of gateways
 	 */
 	List<GCubeGroup> getGateways();
-}
+}
\ No newline at end of file
